﻿////////////////////////////////////////////////////////////
//
//  TpChatGPT
//
//  ChatGPT API ツール
//  選択文字列をChatGPTへ問い合わせて挿入
//
////////////////////////////////////////////////////////////
program TpChatGPT;

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  ShellAPI,
  System.IniFiles,
  TeraPad,
  DelphiChatGPT;

const
  APP_NAME = 'TpChatGPT Ver.0.02';

var
  TpSelStr: string;
  QStr: string;

  SelStrList: TStringList;
  GPTStrList: TStringList;
  SetSelStrList: TStringList;

  IniFile: TIniFile;
  ExePath: string;
  IniFilePath: string;

  Ini_Api_Key: string;
  Ini_Model: string;
  GgtModel: TChatGPTModel;

  // 文字列内の改行を削除する
  function RemoveLineBreaks(const AStr: string): string;
  var
    I: Integer;
  begin
    Result := AStr;
    for I := Length(Result) downto 1 do
    begin
      case Result[I] of
        #13, #10:
          Delete(Result, I, 1);
      end;
    end;
  end;

begin
  if IsTeraPadWindow(TeraPadWindow) then
  begin
    // TeraPadユニット対応か？
    if Copy(TpGetVer(TeraPadWindow), 1, 4) < '0.73' then
    begin
      MessageBox(TeraPadWindow,
                 PChar('TeraPad v0.73 以降が必要です。'),
                 PChar(APP_NAME),
                 MB_OK + MB_DEFBUTTON1 + MB_ICONEXCLAMATION);
      Exit;
    end;

    // INIファイルから設定を読み込む
    ExePath := ParamStr(0);
    IniFilePath := ChangeFileExt(ExePath, '.ini');
    IniFile := TIniFile.Create(IniFilePath);
    try
      Ini_Api_key := IniFile.ReadString('ChatGPT', 'API_KEY', '');
      Ini_Model := IniFile.ReadString('ChatGPT', 'Model', '');
    finally
      IniFile.Free;
    end;

    // Model 設定
    if Ini_Model = '4' then
      GgtModel := GPT4
    else
      GgtModel := GPT3;

    // TeraPadの選択文字列を取得
    TpSelStr := TpGetSelText(TeraPadWindow);
    // 選択文字列内の改行削除してChatGPTへ渡す文字列を取得
    QStr := RemoveLineBreaks(TpSelStr);

   if QStr <> '' then
    begin
      SelStrList := TStringList.Create;
      GPTStrList := TStringList.Create;
      SetSelStrList := TStringList.Create;
      try
        // TeraPadのステータスバーのヒントへ「読み込み中...」を表示
        if GgtModel = GPT4 then
          TpSetHint(TeraPadWindow, 'ChatGPT (gpt-4) 読み込み中...')
        else
          TpSetHint(TeraPadWindow, 'ChatGPT (gpt-3.5-turbo) 読み込み中...');

        // ChatGPTの返答を取得
        GPTStrList.Text := AskChatGPT(GgtModel, Ini_Api_Key, QStr);

        // 「質問＋返答」を変数へ代入
        SelStrList.Text := TpSelStr;
        SetSelStrList.Text := SelStrList.Text +
                              '--' +
                              #13#10 +
                              GPTStrList.Text;

        // TeraPadの選択範囲へ「質問＋返答」をセット
        TpReplaceSel(TeraPadWindow, SetSelStrList.Text);
      finally
        SelStrList.Free;
        GPTStrList.Free;
        SetSelStrList.Free;
        TpSetHint(TeraPadWindow, '');
      end;
    end;
  end
  else
  begin
    MessageBox(GetForegroundWindow,
               PChar('TeraPadの[ツール]から実行して下さい。'),
               PChar(APP_NAME),
               MB_OK + MB_DEFBUTTON1 + MB_ICONEXCLAMATION);
  end;
end.

